using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using KairoEngine.Core;
using KairoEngine.Core.ModuleSystem;
using Sirenix.OdinInspector;

namespace KairoEngine.Achievements
{
    [Serializable, HideReferenceObjectPicker]
    public class AchievementsModule : GameModuleBase
    {
        public override string name => "Achievements Module";
        [FoldoutGroup("@name"), NonSerialized, ShowInInspector, InlineEditor(InlineEditorObjectFieldModes.Boxed)] public AchievementsLibrary library;

        public AchievementsModule(GameConfig config) : base(config)
        {
            this.gameConfig = config;
            this.className = this.GetType().AssemblyQualifiedName;
            this.typeName = "AchievementsModule";
        }

        public override void Load(Transform parent)
        {
            if(library == null)
            {
                Debug.LogError("Missing Achievements Library.\nPlease configure the Achievements module in the Game config file.");
                return;
            }
            var obj = new GameObject();
            obj.transform.parent = parent;
            obj.name = "Achievements";
            var comp = obj.AddComponent<AchievementsController>();
            comp.library = library;
        }

        public override void Reset()
        {
            
        }

        public override void Destroy() { }

        public static AchievementsModule JSONToAchievementsModule(string data)
        {
            try
            {
                return JsonUtility.FromJson<AchievementsModule>(data);
            }
            catch (System.Exception e)
            {
                Debug.LogError($"Could not deserialize AchievementsModule: \n{e}");
                return new AchievementsModule(null);
            }
        }

        public override void OnBeforeSerialize(ObjectSerializer serializer) 
        {
            if(library != null) serializer.AddScriptableObject("Achievements_Library", library);
        }
        public override void OnBeforeDeserialize(ObjectSerializer serializer)
        { 
            library = (AchievementsLibrary)serializer.GetScriptableObject("Achievements_Library");
        }
    }
}